#ifndef POOL_MALLOC_LIBC_OVERRIDE_GCC_AND_WEAK_H_
#define POOL_MALLOC_LIBC_OVERRIDE_GCC_AND_WEAK_H_

#include "poolMalloc.h"

#ifndef __THROW    // I guess we're not on a glibc-like system
# define __THROW   // __THROW is just an optimization, so ok to make it ""
#endif

#ifndef __GNUC__
# error libc_override_gcc_and_weak.h is for gcc distributions only.
#endif

#define ALIAS(pool_fn)   __attribute__ ((alias (#pool_fn)))

void* operator new(size_t size) throw (std::bad_alloc)
    ALIAS(pool_new);
void operator delete(void* p) __THROW
    ALIAS(pool_delete);
void* operator new[](size_t size) throw (std::bad_alloc)
    ALIAS(pool_newarray);
void operator delete[](void* p) __THROW
    ALIAS(pool_deletearray);
void* operator new(size_t size, const std::nothrow_t& nt) __THROW
    ALIAS(pool_new_nothrow);
void* operator new[](size_t size, const std::nothrow_t& nt) __THROW
    ALIAS(pool_newarray_nothrow);
void operator delete(void* p, const std::nothrow_t& nt) __THROW
    ALIAS(pool_delete_nothrow);
void operator delete[](void* p, const std::nothrow_t& nt) __THROW
    ALIAS(pool_deletearray_nothrow);

extern "C" {
  void* malloc(size_t size) __THROW               ALIAS(pool_malloc);
  void free(void* ptr) __THROW                    ALIAS(pool_free);
  void* realloc(void* ptr, size_t size) __THROW   ALIAS(pool_realloc);
  void* calloc(size_t n, size_t size) __THROW     ALIAS(pool_calloc);
  void cfree(void* ptr) __THROW                   ALIAS(pool_cfree);
  void* memalign(size_t align, size_t s) __THROW  ALIAS(pool_memalign);
  void* valloc(size_t size) __THROW               ALIAS(pool_valloc);
  void* pvalloc(size_t size) __THROW              ALIAS(pool_pvalloc);
  int posix_memalign(void** r, size_t a, size_t s) __THROW
      ALIAS(pool_posix_memalign);
  void malloc_stats(void) __THROW                 ALIAS(pool_malloc_stats);
  int mallopt(int cmd, int value) __THROW         ALIAS(pool_mallopt);
#ifdef HAVE_STRUCT_MALLINFO
  struct mallinfo mallinfo(void) __THROW          ALIAS(pool_mallinfo);
#endif
  size_t malloc_size(void* p) __THROW             ALIAS(pool_malloc_size);
  size_t malloc_usable_size(void* p) __THROW      ALIAS(pool_malloc_size);
}   // extern "C"

#undef ALIAS

// No need to do anything at tcmalloc-registration time: we do it all
// via overriding weak symbols (at link time).
static void ReplaceSystemAlloc() { }

#endif  // TCMALLOC_LIBC_OVERRIDE_GCC_AND_WEAK_INL_H_
