#!/bin/bash
set -o xtrace

# This points to the top-level directoy of the tar-ball
ROOT=""

# Setup benchmark inputs and run directory
mkdir myApp
cd myApp

###############################################

# Run the application using inscount.so pinTool to get an initial list of all callpoints.
export PM_MAPPINGSFILE=$ROOT/tests/emptyTable
export PM_MODE=0
export PM_SIGNATURE=0
export PM_TRACEDEPTH=2
export PM_RELADDR=1
export PIN_LD_RESTORE_REQUIRED=t
export PIN_APP_LD_PRELOAD=$ROOT/poolMalloc/libpoolmalloc.so
export PM_OUTPUTFILE=./.temp

pin -ifeellucky -t $ROOT/pinTools/obj-intel64/inscount.so -start 20000000000 -length 10000000000 -- myApp myApp-input

# This order is important
unset PM_OUTPUTFILE
unset LD_PRELOAD
unset PM_MAPPINGSFILE
unset PIN_LD_RESTORE_REQUIRED
unset PIN_APP_LD_PRELOAD
unset PM_MODE
unset PM_SIGNATURE
unset PM_TRACEDEPTH
unset PM_RELADDR

# Pre-process the output of inscount.so to get a filtered list of callpoints, with a maximum of 128 callpoints.
rm ./.temp
sort -n -r -k2 inscount.out -o inscount.out
$ROOT/whirltool/procHashList.py inscount.out filtered-callpoints 128 0.00001

######################################

# Now profile the application one more time to gather missCurves
export PM_MAPPINGSFILE=./filtered-callpoints
export PM_MODE=1
export PM_SIGNATURE=0
export PM_TRACEDEPTH=2
export PM_RELADDR=1
export PIN_LD_RESTORE_REQUIRED=t
export PIN_APP_LD_PRELOAD=$ROOT/poolMalloc/libpoolmalloc.so

pin -t $ROOT/pinTools/obj-intel64/memoryAccessProfiler.so -start 20000000000 -length 10000000000 -- myApp myApp-input

# this order is important
unset PM_OUTPUTFILE
unset LD_PRELOAD
unset PM_MAPPINGSFILE
unset PIN_LD_RESTORE_REQUIRED
unset PIN_APP_LD_PRELOAD
unset PM_MODE
unset PM_SIGNATURE
unset PM_TRACEDEPTH
unset PM_RELADDR

# Run the clustering algorithm to get a final list of callpoints
$ROOT/whirltool/whirltool.py missCurves.h5 filtered-callpoints $ROOT/tests/results/ -t 0.0001 --llcSize 64M --threadPoolComesFirst

#####################################

# Finish
cd ../

