'''
/** $lic$
 * Copyright (C) 2015-2016 by Massachusetts Institute of Technology
 *
 * This file is part of Whirltool.
 *
 * Whirltool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * If you use this software in your research, we request that you reference
 * the Whirlpool paper ("Whirlpool: Improving Dynamic Cache Management with Static Data
 * Classification", Mukkara, Beckmann and Sanchez, ASPLOS-21, April 2016) as the
 * source in any publications that use this software, and that you send us a
 * citation of your work.
 *
 * Whirltool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Whirltool.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/
'''

import argparse, sys, os

parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)

parser.add_argument("inputFile", type=str, default="",  help="Input *.h5 file containing misscurves")
parser.add_argument("mappingsFile", type=str, default="",  help="List of all unclustered callpoints")
parser.add_argument("outputDir", type=str, default="",  help="Output dir where callpoint maps are written")

parser.add_argument("--threshold", "-t", type=float, default=0.0001, help="Discard threshold")
parser.add_argument("--threadPoolComesFirst", action="store_true", default=False, help="Thread pool corresponds to the first set of misscurves.\
            use this when the profiler didn't use fastforwarding. By default, fastforwarding is used and thread pool comes last")

parser.add_argument("--llcSize", "-l", type=str, default="64M", help="LLC size eg. 512K or 64M ")

args = parser.parse_args()

def InitGraphsDir(graphsDir, silent=False):
    if not silent:
        print "Graphs will be saved in " + graphsDir,
    if not os.path.exists(graphsDir):
        os.makedirs(graphsDir)
        if not silent:
            print "(created)"
    else:
        if not silent:
            print "(existing)"
    f = open(os.path.join(graphsDir, "cmd.txt"), 'wa')
    print >>f, "Command line:", " ".join(sys.argv)
    f.close()

def sizeInMB(sizeString):
    if sizeString[-1] == 'M':
        return float(sizeString[0:-1])
    elif sizeString[-1] == 'K' :
        return float(sizeString[0:-1])/1024
    else :
        print 'Bad size string: ' + sizeString
        sys.exit()

tableau_colors = (
    (114/255., 158/255., 206/255.),
    (255/255., 158/255.,  74/255.),
    (103/255., 191/255.,  92/255.),
    (237/255., 102/255.,  93/255.),
    (173/255., 139/255., 201/255.),
    (168/255., 120/255., 110/255.),
    (237/255., 151/255., 202/255.),
    (162/255., 162/255., 162/255.),
    (205/255., 204/255.,  93/255.),
    (109/255., 204/255., 218/255.),
    (144/255., 169/255., 202/255.),
    (225/255., 157/255.,  90/255.),
    (122/255., 193/255., 108/255.),
    (225/255., 122/255., 120/255.),
    (197/255., 176/255., 213/255.),
    (196/255., 156/255., 148/255.),
    (247/255., 182/255., 210/255.),
    (199/255., 199/255., 199/255.),
    (219/255., 219/255., 141/255.),
    (158/255., 218/255., 229/255.),
    (198/255., 118/255., 255/255.),
    (58/255., 208/255., 129/255.),
)
