'''
/** $lic$
 * Copyright (C) 2015-2016 by Massachusetts Institute of Technology
 *
 * This file is part of Whirltool.
 *
 * Whirltool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * If you use this software in your research, we request that you reference
 * the Whirlpool paper ("Whirlpool: Improving Dynamic Cache Management with Static Data
 * Classification", Mukkara, Beckmann and Sanchez, ASPLOS-21, April 2016) as the
 * source in any publications that use this software, and that you send us a
 * citation of your work.
 *
 * Whirltool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Whirltool.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/
'''

import os, math
import matplotlib.pyplot as plt
import numpy as np
from baseScript import args

def cmd(c):
    return os.popen(c).read()

####################################################
# Reading input files, preprocessing output and writing out final output

def printClusters(fclusterdata, highAPKIindices):
    print "pool -> [callpoint number]"
    for i in range(1, int(np.max(fclusterdata)) + 1):
        print i, ' ', [highAPKIindices[pool] for pool in np.where(fclusterdata==i)[0]]

def readPcTableFile(pcTable):
    pcList = []
    poolList = []
    for line in open(pcTable):
        if len(line.strip().split(" ")) == 2:
            (pc, pool) = line.strip().split(" ")
            pcList.append(pc)
            poolList.append(pool)
    assert(len(poolList) == len(set(poolList)))
    return pcList, poolList

def optimizePools(highAPKIindices, numPools, fclusterdata, cumAccesses):
    numClusters = max(fclusterdata)

    # threadPool position in the list of starting pools
    if args.threadPoolComesFirst:
        threadPoolPos = 0
    else:
        threadPoolPos = numPools

    index = [i for i, j in enumerate(highAPKIindices) if j == threadPoolPos]
    if len(index) > 0:
        assert(len(index) == 1)
        index = index[0]
        clusterWithThreadPool = fclusterdata[index]
    else:
        clusterWithThreadPool = -1

    if clusterWithThreadPool != -1:
        # thread share is in one of the clusters
        # remove the cluster containing thread pool from the list of clusters
        # i.e. give all callpoints a pool number of 0
        for i in range(len(fclusterdata)):
            if fclusterdata[i] == clusterWithThreadPool:
                fclusterdata[i] = 0
            if fclusterdata[i] > clusterWithThreadPool:
                fclusterdata[i] -= 1
    else:
        # thread pool had very low APKI to begin with
        # so, combine the smallest APKI cluster into thread pool
        lowAPKIpools = cumAccesses < args.threshold

        cumClusterApki = np.zeros(numClusters)
        for i in range(len(fclusterdata)):
            cumClusterApki[fclusterdata[i] - 1] += cumAccesses[highAPKIindices[i]]

        numShares = cumAccesses.shape[0]
        discardedPoolApki = sum([cumAccesses[i] for i in range(numShares) if lowAPKIpools[i]])
        print "discardedPoolApki", discardedPoolApki

        if discardedPoolApki < 0.05:
            lowestAPKIcluster = np.argmin(cumClusterApki) + 1
            for i in range(len(fclusterdata)):
                if fclusterdata[i] == lowestAPKIcluster:
                    fclusterdata[i] = 0
                if fclusterdata[i] > lowestAPKIcluster:
                    fclusterdata[i] -= 1

    return fclusterdata

def genPcTableFile(fclusterdata, highAPKIindices, pcList):
    assert(len(highAPKIindices) == len(fclusterdata))

    finalPools = len(set(fclusterdata)) - (1 if 0 in fclusterdata else 0)
    finalPCs = sum(fclusterdata >= 0)
    assert(finalPCs == len(fclusterdata))

    if args.threadPoolComesFirst:
        threadPoolPos = 0
        userShareOffset = 1
    else:
        threadPoolPos = len(pcList)
        userShareOffset = 0

    if threadPoolPos in highAPKIindices:
        finalPCs -= 1

    print "Generating output pcTable file"

    out = "%d\n" % finalPools
    out += "%d\n" % finalPCs
    for i in range(len(highAPKIindices)):
        if highAPKIindices[i] != threadPoolPos: # no need to list thread share
            out += "%s %d\n" % (pcList[highAPKIindices[i] - userShareOffset],\
                    fclusterdata[i])

    print out
    return out

def writePcTableFile(out, clusters):
    destDir = args.outputDir
    destFile = destDir + "/" + "hashTable-%d" % (clusters - 1)
    open(destFile, "w").write(out)

def writeClusterDistances(clusterDistances):
    destDir = args.outputDir
    destFile = destDir + "/" + "hashTable-clusterDistances"
    np.save(destFile, clusterDistances)
